package payment;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import com.atomikos.icatch.HeurCommitException;
import com.atomikos.icatch.HeurRollbackException;
import com.atomikos.tcc.TccException;
import com.atomikos.tcc.TccService;
import com.atomikos.tcc.TccServiceManager;

public class PaymentTccService implements TccService 
{
	
	
	private TccServiceManager tccmgr;
	private long timeout;
	
	public PaymentTccService ( TccServiceManager tccmgr ,
			long timeout )
	{
		this.tccmgr = tccmgr;
		this.timeout = timeout;
	}
	
	
	/**
	 * Try a payment as a TCC job. 
	 * 
	 * @param cardNo
	 * @param amount
	 * @throws Exception
	 */
	public void tryPayment ( String cardNo , int amount ) 
	throws Exception
	{
		//start (register) the work for cancel/confirm
		//and obtain the id to use as reference
		String id = tccmgr.register ( this , timeout );
		
		
		try {
			//detect if card no is illegal or not
			if ( cardNo.startsWith ( "fake" ) )
					throw new IllegalArgumentException ( "Wrong card no: " + cardNo );
			
			//perform the work (i.e., the business logic)
			//the application should use the id to 
			//identify the work for later confirm/cancel
			Connection conn = PaymentDbUtils.getConnection();
			Statement s = conn.createStatement();
			s.executeUpdate ( "insert into PAYMENTS values ( " +
					" '" + id + "' , '" + cardNo + "' ," + amount + "," +
					" 'PENDING' )" );
			System.out.println ( "CREATED PAYMENT: " + id );
			conn.close();
			//mark the work as completed in the system
			//to trigger the cancel/confirm callbacks
			tccmgr.completed ( id );
			
		} catch ( Exception e ) {
			//e.printStackTrace();
			//on any exception: mark the work as failed
			//so cancel is the only possible callback
			tccmgr.failed ( id );
			
			//re-throw e to make overall task cancel
			//(otherwise, nested semantics will hide this failure!)
			throw e;
		}
	}
	
	
	public void confirm ( String id ) 
	throws HeurRollbackException, TccException 
	{
		
		try {
			Connection conn = PaymentDbUtils.getConnection();
			Statement s = conn.createStatement();
			s.executeUpdate ( 
					"update PAYMENTS set status = 'CONFIRMED' where key = '" + id +"'" );
			System.out.println ( "CONFIRMED PAYMENT: " + id );
			conn.close();		
		} catch ( SQLException e ) {
			e.printStackTrace();
			//optional: throw TccException to retry confirmation
			throw new TccException();
		}

	}

	public void cancel ( String id ) 
	throws HeurCommitException, TccException 
	{
		try {
			Connection conn = PaymentDbUtils.getConnection();
			Statement s = conn.createStatement();
			//set status to canceled; alternatively, we could also
			//choose to delete the payment row instead; this depends
			//on the business-specific model of cancelation
			s.executeUpdate ( 
					"update PAYMENTS set status = 'CANCELED' where key = '" + id +"'" );
			System.out.println ( "CANCELLED PAYMENT: " + id );
			conn.close();		
		} catch ( SQLException e ) {
			e.printStackTrace();
			//optional: throw TccException to retry cancelation
			throw new TccException();
		}

	}

	public boolean recover ( String id ) 
	{
		//check if the id is in the database;
		//return true if so
		boolean ret = false;
		
		Connection conn = null;
		try {
            conn = PaymentDbUtils.getConnection();
            Statement s = conn.createStatement();
            String sql = "select * from Payments where key='"
                                + id +"'";
            ResultSet rs = s.executeQuery ( sql );
            if ( rs.next() ) {
            	//the record with the specified id exists -> this is our work
            	ret = true;
            }
            conn.close();
		}
		catch ( Exception e ) {
			e.printStackTrace();
		}
		
		return ret;
	}
	
	/**
	 * Read-only method to get the payments for a card.
	 * This method is NOT transactional nor TCC. 
	 * 
	 * @param cardno
	 * @return
	 * @throws Exception
	 */
	public int getPayments ( String cardno )
	throws Exception
	{
		int result = 0;
		Connection conn = null;
		try {
            conn = PaymentDbUtils.getConnection();
            Statement s = conn.createStatement();
            String sql = "select amount from Payments where cardno='"
                                + cardno+"'";
            ResultSet rs = s.executeQuery ( sql );
            if ( rs.next() )
            		result = rs.getInt ( 1 );
            s.close();
		}
		finally {
			if ( conn != null ) conn.close();
		}
		return result;
	}


}
