package portal;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * 
 * Utility class with datbase-related code for the portal
 * service. This class uses the DriverManager for JDBC, to 
 * illustrate that non-JTA/XA datasources can be used for 
 * TCC transactions. For better performance, however, a pooling 
 * DataSource should be used instead. 
 *
 */

public class PortalDbUtils 
{
    //the user name in the database; change if needed.
    private static String user = "sa";

    //the password for the user; change if needed.
    //the current settings are empty strings, for HypersonicSQL
    private static String passwd = "";

    //the full name of the JDBC driver class
    //change if required
    private static String driverClassName = "org.hsqldb.jdbcDriver";

    //the URL to connect with; this should be a valid DriverManager URL
    //change if needed
    private static String connectUrl = "jdbc:hsqldb:file:TCCPortalDB";
    
    /**
     * Gets a connection to the database.
     * @return The connection. 
     * @throws SQLException On errors.
     */
    
    public static Connection getConnection() throws SQLException
    {
    		Connection ret = null;
		try {
			Class.forName(driverClassName).newInstance();
		} catch (InstantiationException e) {
			throw new SQLException(
					"Could not instantiate driver class: "
					+ driverClassName);
		} catch (IllegalAccessException e) {
			throw new SQLException(e.getMessage());
		} catch (ClassNotFoundException e) {
			throw new SQLException(
					"Driver class not found: " + driverClassName);
		}

		ret = DriverManager.getConnection ( connectUrl, user, passwd );

		return ret;
    }
    
    public static void createDatabase()
    throws SQLException
    {
    		Connection conn = getConnection();
		Statement s = conn.createStatement();

		try {
            s.executeQuery ( "select * from Stock" );
        }
        catch ( SQLException ex ) {
            //table not there => create it
            System.err.println ( "Creating STOCK table..." );
            s.executeUpdate ( "create table STOCK ( " +
                              " itemId DECIMAL ( 19 ,0 ), amount DECIMAL (19,0) , " +
                              " unitPrice DECIMAL ( 19, 0 ) )" );
            for ( int i = 0; i < 100 ; i++ ) {
                s.executeUpdate ( "insert into STOCK values ( " +
                                  i  + ", 1000 , " + i + " )" );
            }
            
            System.err.println ( "Creating PURCHASES table..." );
            s.executeUpdate ( "create table PURCHASES ( key VARCHAR ( 100 ) , itemId DECIMAL ( 19 ,0 ), quantity DECIMAL (19,0) , status VARCHAR(20) )" );
            
        }


		conn.close();
    }

}
