package portal;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;



import com.atomikos.icatch.HeurCommitException;
import com.atomikos.icatch.HeurRollbackException;
import com.atomikos.tcc.TccException;
import com.atomikos.tcc.TccService;
import com.atomikos.tcc.TccServiceManager;

public class PortalTccService implements TccService 
{
	private TccServiceManager tccmgr;
	private long timeout;
	
	public PortalTccService ( TccServiceManager tccmgr ,
			long timeout )
	{
		this.tccmgr = tccmgr;
		this.timeout = timeout;
	}
	
	/**
	 * Try a purchase as a TCC job. 
	 * This way, the purchase result is subject to
	 * global termination of the distributed work.
	 * Note: this method will start and 
	 * complete its own (sub)activity, meaning
	 * for distributed work it should be 
	 * called with an existing activity context.
	 * 
	 * @param itemId
	 * @param qty
	 * @return the price of the purchase
	 * @throws Exception
	 */
	public int tryPurchase ( int itemId , int qty ) 
	throws Exception
	{
		//start (register) the work for cancel/confirm
		//and obtain the id to use as reference
		String id = tccmgr.register ( this , timeout );
		int price = 0;
		
		try {
			//perform the work (i.e., the business logic)
			//the application should use the id to 
			//identify the work for later confirm/cancel
			Connection conn = PortalDbUtils.getConnection();
			Statement s = conn.createStatement();
			String sql = "select unitPrice from Stock " +
            " where itemId = " + itemId;
			ResultSet rs = s.executeQuery ( sql );
			if ( rs == null || !rs.next() ) 
              throw new Exception ( "Item not found: " +itemId );
			price = rs.getInt ( 1 );
			
			//decrement stock already, to avoid availability problems during confirm
			s.executeUpdate ( "update STOCK set amount = amount - " + qty + " where itemId = '" + itemId + "'"  );
			s.executeUpdate ( "insert into PURCHASES values ( '" + id + "' , " +
					itemId + " , " + qty + " , 'PENDING' )" );
			conn.close();
			System.out.println ( "CREATED PURCHASE WITH ID: " + id );
			//mark the work as completed in the system
			//to trigger the cancel/confirm callbacks
			tccmgr.completed ( id );
			
		} catch ( Exception e ) {
			e.printStackTrace();
			//on any exception: mark the work as failed
			//so cancel is the only possible callback
			tccmgr.failed ( id );
			
			//rethrow; otherwise nested semantics will mask
			//this failure and allow global commit!
			throw e;
		}
		return price;
	}

	
	public void confirm ( String id ) 
	throws HeurRollbackException, TccException 
	{
		try {
			Connection conn = PortalDbUtils.getConnection();
			Statement s = conn.createStatement();
			s.executeUpdate ( 
					"update PURCHASES set status = 'CONFIRMED' where key = '" + id +"'" );
			conn.close();		
			System.out.println ( "CONFIRMED PURCHASE WITH ID: " + id );
		} catch ( SQLException e ) {
			e.printStackTrace();
			//optional: throw TccException to retry confirmation
			throw new TccException();
		}

	}

	public void cancel ( String id ) 
	throws HeurCommitException, TccException 
	{
		try {
			Connection conn = PortalDbUtils.getConnection();
			Statement s = conn.createStatement();
			//return the items of the canceled it to the stock
			int qty = 0;
			int itemId = 0;
			//remember: cancel can be called twice or more -> check for pending status
			//or stock will be returned multiple times
			ResultSet rs = s.executeQuery ( "select itemId,quantity from PURCHASES where key = '" + id +"' and status = 'PENDING'" );
			if ( rs.next() ) {
				//if status is no longer PENDING then this won't be done
				itemId = rs.getInt ( 1 );
				qty = rs.getInt ( 2 );
				s.executeUpdate ( "update STOCK set amount = amount + " + qty + " where itemId = '" + itemId + "'"  );
			}
			
			//set status to canceled; alternatively, we could also
			//choose to delete the payment row instead; this depends
			//on the business-specific model of cancelation
			s.executeUpdate ( 
					"update PURCHASES set status = 'CANCELED' where key = '" + id +"'" );
			System.out.println ( "CANCELLED PURCHASE WITH ID: " + id );
			conn.close();		
		} catch ( SQLException e ) {
			e.printStackTrace();
			//optional: throw TccException to retry cancelation
			throw new TccException();
		}
	}

	public boolean recover ( String id ) 
	{
		//check if the id is in the database;
		//return true if so
		boolean ret = false;
		
		Connection conn = null;
		try {
            conn = PortalDbUtils.getConnection();
            Statement s = conn.createStatement();
            String sql = "select * from PURCHASES where key='"
                                + id +"'";
            ResultSet rs = s.executeQuery ( sql );
            if ( rs.next() ) {
            	//the record with the specified id exists -> this is our work
            	ret = true;
            }
            conn.close();
		}
		catch ( Exception e ) {
			e.printStackTrace();
		}
		
		return ret;
	}
	
	
	/**
	 * Utility method to get remaining stock.
	 * This method does not execute as a TCC
	 * distributed transaction (it is purely local).
	 * 
	 * @param itemId
	 * @return
	 * @throws Exception
	 */
	public int getStock ( int itemId )
	throws Exception
	{
		int result = 0;
		Connection conn = null;
		
		try {
            conn = PortalDbUtils.getConnection();
            Statement s = conn.createStatement();
            String sql = "select amount from Stock " +
                " where itemId = " + itemId;
            ResultSet rs = s.executeQuery ( sql );
            if ( rs == null || !rs.next() ) 
                  throw new Exception ( "Item not found: " +itemId );
            result = rs.getInt ( 1 );
            
            s.close();
		}
		finally {
			if ( conn != null ) conn.close();
		}
		
		return result;
	}
	

}
